@@ -47,9 +47,11 @@ import sys
 import pandas as pd
 
 from datetime import datetime
+from http.client import IncompleteRead
 from typing import List, Optional
 
 from Bio import Entrez
+from Bio.Entrez.Parser import NotXMLError
 from saintBioutils.utilities.logger import config_logger
 from saintBioutils.utilities import file_io
 from saintBioutils.utilities.file_io import make_output_directory
@@ -60,6 +62,10 @@ from cazy_webscraper import closing_message
 from cazy_webscraper.cazy_scraper import connect_existing_db
 from cazy_webscraper.expand import get_chunks_list
 from cazy_webscraper.ncbi import post_ids
+from cazy_webscraper.ncbi.taxonomy.lineage import (
+    retry_tax_retrieval,
+    get_lineage_data,
+)
 from cazy_webscraper.sql import sql_orm, sql_interface
 from cazy_webscraper.sql.sql_interface.add_data.add_ncbi_tax_data import (
     add_ncbi_taxonomies,
@@ -115,7 +121,7 @@ def main(argv: Optional[List[str]] = None, logger: Optional[logging.Logger] = No
             config_dict,
             kingdom_filters,
             taxonomy_filter_dict,
-            set(),  # ec_filters not applied when scraping CAZy
+            ec_filters,  # ec_filters not applied when scraping CAZy
             'NCBI Taxonomy',
             'NCBI taxonomc lineages',
             session,
@@ -131,7 +137,9 @@ def main(argv: Optional[List[str]] = None, logger: Optional[logging.Logger] = No
         cache_dir = cache_dir / "ncbi_tax_retrieval"
         make_output_directory(cache_dir, args.force, args.nodelete_cache)
 
+    # Use Cache
     if args.use_lineage_cache is not None:
+        gbk_dict, tax_ids, prot_id_dict = None, None, None
         logger.info("Adding cached lineages to local CAZyme db")
         try:
             with open(args.use_lineage_cache, "r") as fh:
@@ -145,6 +153,7 @@ def main(argv: Optional[List[str]] = None, logger: Optional[logging.Logger] = No
             )
             sys.exit(1)
 
+    # Don't use cache
     else:
         gbk_dict = get_db_proteins(
             class_filters,
@@ -272,92 +281,97 @@ def get_ncbi_ids(gbk_dict, cache_dir, args):
 
     Return set of NCBI Tax IDs, and dict of NCBI Prot ID valued by Gbk sequence accession
     """
+    tax_ids, prot_id_dict = set(), {}
     logger = logging.getLogger(__name__)
 
     if (args.use_tax_ids is not None) and (args.use_protein_ids is not None):
-        logger.info("Using only cached NCBI Tax and Protein IDs")
-        try:
-            with open(args.use_tax_ids, "r") as fh:
-                tax_ids = [_.replace("\n", "") for _ in fh.read().splitlines() if _ != "\n"]
-        except FileNotFoundError:
-            logger.error(
-                f"Could not find Tax ID cache at:\n{str(args.use_tax_ids)}\n"
-                "Check path is correct\n"
-                "Terminating program"
-            )
-            sys.exit(1)
-
-        try:
-            with open(args.use_protein_ids, "r") as fh:
-                cached_prot_data = fh.read().splitlines()
-        except FileNotFoundError:
-            logger.error(
-                f"Could not find NCBI Protein ID cache at:\n{str(args.use_protein_ids)}\n"
-                "Check path is correct\n"
-                "Terminating program"
-            )
-            sys.exit(1)
-
-        prot_id_dict = {}
-        for line in cached_prot_data:
-            if line == "\n":
-                continue
-            prot_id_dict[line.split("\t")[0].strip()] = line.split("\t")[1].strip().replace("\n", "")
+        # only use ids in files
+        tax_ids = load_tax_ids(args.use_tax_ids)
+        prot_id_dict = load_protein_ids(args.use_protein_ids)
 
         return tax_ids, prot_id_dict
 
-    else:
-        tax_ids, prot_id_dict = get_ncbi_tax_prot_ids(list(gbk_dict.keys()), cache_dir, args)
+    tax_ids, prot_id_dict = get_ncbi_tax_prot_ids(list(gbk_dict.keys()), cache_dir, args)
 
-        if args.use_tax_ids is not None:
-            logger.info("Using cached NCBI Tax IDs")
-            try:
-                with open(args.use_tax_ids, "r") as fh:
-                    tax_ids = [_.replace("\n", "") for _ in fh.read().splitlines() if _ != "\n"]
-            except FileNotFoundError:
-                logger.error(
-                    f"Could not find Tax ID cache at:\n{str(args.use_tax_ids)}\n"
-                    "Check path is correct\n"
-                    "Terminating program"
-                )
-                sys.exit(1)
+    if args.use_tax_ids is not None:
+        tax_ids = load_tax_ids(args.use_tax_ids)
 
-        if args.use_protein_ids is not None:
-            logger.info("Using cached NCBI Protein IDs")
-            try:
-                with open(args.use_protein_ids, "r") as fh:
-                    cached_prot_data = fh.read().splitlines()
-            except FileNotFoundError:
-                logger.error(
-                    f"Could not find NCBI Protein ID cache at:\n{str(args.use_protein_ids)}\n"
-                    "Check path is correct\n"
-                    "Terminating program"
-                )
-                sys.exit(1)
+    if args.use_protein_ids is not None:
+        prot_id_dict = load_protein_ids(args.use_protein_ids)
 
-            prot_id_dict = {}
-            for line in cached_prot_data:
-                if line == "\n":
-                    continue
-                prot_id_dict[line.split("\t")[0].strip()] = line.split("\t")[1].strip().replace("\n", "")
-
-    # Returns a set of NCBI Tax ids and dict {ncbi prot id: prot acc}
-
-    logger.info("Logging retrieved NCBI Taxonomy and Protein IDs")
+    logger.info("caching retrieved NCBI Taxonomy and Protein IDs")
     with open((cache_dir/"tax_ids.out"), "a") as fh:
         for tax_id in tax_ids:
             fh.write(f"{tax_id}\n")
 
-    with open((cache_dir/"protein_ncbi_ids.out"), "a") as fh:
-        for ncbi_prot_id in prot_id_dict:
-            fh.write(f"{ncbi_prot_id}\t{prot_id_dict[ncbi_prot_id]}\n")
+    if prot_id_dict is not None:
+        with open((cache_dir/"protein_ncbi_ids.out"), "a") as fh:
+            for ncbi_prot_id in prot_id_dict:
+                fh.write(f"{ncbi_prot_id}\t{prot_id_dict[ncbi_prot_id]}\n")
 
     return tax_ids, prot_id_dict
 
 
+def load_tax_ids(tax_id_path):
+    """Load NCBI taxonomy ids from plain text file
+
+    :param tax_id_path: Path, path to file of tax ids
+
+    Return list of tax ids
+    """
+    logger = logging.getLogger(__name__)
+    logger.warning(f"Use Tax IDs in {tax_id_path}")
+    try:
+        with open(tax_id_path, "r") as fh:
+            tax_ids = [_.replace("\n", "") for _ in fh.read().splitlines() if _ != "\n"]
+    except FileNotFoundError:
+        logger.error(
+            f"Could not find Tax ID cache at:\n{str(tax_id_path)}\n"
+            "Check path is correct\n"
+            "Terminating program"
+        )
+        sys.exit(1)
+
+    return tax_ids
+
+
+def load_protein_ids(prot_id_path):
+    """Load protein IDs and accessions from tab delimited file
+
+    :param prot_id_path: Path, path to tab delimited file
+
+    Return dict {ID: accession}
+    """
+    logger = logging.getLogger(__name__)
+    logger.warning(f"Use Protein IDs and accessions in {prot_id_path}")
+    try:
+        with open(prot_id_path, "r") as fh:
+            cached_prot_data = fh.read().splitlines()
+    except FileNotFoundError:
+        logger.error(
+            f"Could not find NCBI Protein ID cache at:\n{str(prot_id_path)}\n"
+            "Check path is correct\n"
+            "Terminating program"
+        )
+        sys.exit(1)
+
+    prot_id_dict = {}
+    for line in cached_prot_data:
+        if line == "\n":
+            continue
+        prot_id_dict[line.split("\t")[0].strip()] = line.split("\t")[1].strip().replace("\n", "")
+
+    prot_id_dict
+
+
 def get_ncbi_tax_prot_ids(protein_accessions, cache_dir, args):
     """Retrieve NCBI Taxonomy and Protein database IDs
 
+    Post the protein accession.
+    Then entrez.efetch the protein records to associate each accession with the correct id
+    Then entrez.elink to retrieve all linked taxonomy records - do no associated with the correct 
+    protein accesion at this point.
+
     :param protein_accessions: list of NCBI protein version accessions
     :param cache_dir: Path, path to cache directory
     :param args: cmd-line args parser
@@ -369,155 +383,185 @@ def get_ncbi_tax_prot_ids(protein_accessions, cache_dir, args):
     # break up list into batches
     batches = get_chunks_list(protein_accessions, args.batch_size)
 
-    failed_batches = {}  # {batch: # of tries}
+    failed_batches = {}  # {protein: tries}
 
-    tax_ids = set()
+    tax_ids = set()  # set of all retrieve tax ids
 
-    protein_ncbi_ids = {}
+    protein_ncbi_ids = {}  # {ncdi prot id : ncbi prot acc}
 
-    for batch in tqdm(batches, desc="Getting NCBI Tax IDs"):
+    for batch in tqdm(batches, desc="Getting NCBI Protein and Taxonomy IDs"):
+        # post the IDs
+        try:
+            query_key, web_env = post_ids(batch, 'Protein', args)
+        except RuntimeError as err:
+            logger.warning(
+                "The batch of protein accessions contains an accession not in NCBI\n"
+                "Will parse the accessions individually later to identify the offending accession\n"
+                f"Batch:\n{batch}"
+            )
+            for protein in batch:
+                failed_batches[protein] = 0
+            continue
+        
+        if query_key is None:
+            for protein in batch:
+                failed_batches[protein] = 0
+            continue
+        
+        # fetch protein ids - associate prot id with prot acc
+        new_protein_ids = get_prot_ids(query_key, web_env, args)
 
-        new_tax_ids, new_prot_ids, failed_batches = get_ids_from_ncbi(batch, args, failed_batches)
+        if new_protein_ids is None:
+            for protein in batch:
+                failed_batches[protein] = 0
+            continue
 
+        protein_ncbi_ids.update(new_protein_ids)
+
+        new_tax_ids = link_prot_taxs(query_key, web_env, args, batch)
+
+        if new_tax_ids is None:
+            for protein in batch:
+                failed_batches[protein] = 0
+            continue
+        
         tax_ids = tax_ids.union(new_tax_ids)
-        protein_ncbi_ids.update(new_prot_ids)
 
-    failed_individuals = {}  # prot_acc: tries
-
-    if len(failed_batches.keys()) != 0:
-        for batch in tqdm(failed_batches, "Retrying failed batches"):
-            batch_proteins = failed_batches[batch]['proteins']
-            for prot in batch_proteins:
-                new_tax_ids, new_prot_ids, failed_individuals = get_ids_from_ncbi(
-                    [prot],
-                    args,
-                    failed_individuals,
-                )
-
-                tax_ids = tax_ids.union(new_tax_ids)
-                protein_ncbi_ids.update(new_prot_ids)
-
-    failed_proteins = set()
-
-    if len(list(failed_individuals.keys())) != 0:
-        prots_to_retry = [failed_individuals[_]['proteins'] for _ in failed_individuals]
-
-        logger.warning(f"Retrying individual failed protein IDs")
-
-        while len(list(failed_individuals.keys())) > 0:
-            for prot in prots_to_retry:
+    # retry failed batches
+    if len(list(failed_batches.keys())) != 0:
+        logger.warning(
+            f"Retrying to retrieve Protein IDs and Taxonomy IDs for {len(list(failed_batches.keys()))} proteins"
+        )
+        while len(list(failed_batches.keys())):
+            for protein in failed_batches:
+                # post ids
                 try:
-                    failed_individuals[prot]
-                except KeyError:
+                    query_key, web_env = post_ids([protein], 'Protein', args)
+                except RuntimeError as err:
+                    logger.warning(
+                        f"The protein accessions {protein} is not listed in NCBI\n"
+                        "Therefore not retrieve taxonomy data for this protein.\n"
+                        f"Error:\n{err}"
+                    )
+                    del failed_batches[protein]
+                    continue
+                
+                if query_key is None:
+                    failed_batches[protein] += 1
+                    if failed_batches[protein] >= args.retries:
+                        logger.warning(
+                            f"Ran out of connection reattempt for protein accession {protein} while posting IDs\n"
+                            "Therefore not retrieve taxonomy data for this protein."
+                        )
+                        del failed_batches[protein]
                     continue
-                except TypeError:
-                    try:
-                        failed_individuals[str(prot)]
-                    except KeyError:
-                        continue
 
-                new_tax_ids, new_prot_ids, failed_individuals = get_ids_from_ncbi(
-                    prot,
-                    args,
-                    failed_individuals,
-                )
+                # fetch prot ids - linked to acc
+                new_protein_ids = get_prot_ids(query_key, web_env, args)
 
+                if new_protein_ids is None:
+                    failed_batches[protein] += 1
+                    if failed_batches[protein] >= args.retries:
+                        logger.warning(
+                            f"Ran out of connection reattempt for protein accession {protein} while fetching Protein IDs\n"
+                            "Therefore not retrieve taxonomy data for this protein."
+                        )
+                        del failed_batches[protein]
+                    continue
+
+                protein_ncbi_ids.update(new_protein_ids)
+
+                # fetch linked tax ids - not linked to acc
+                new_tax_ids = link_prot_taxs(query_key, web_env, args, batch)
+
+                if new_tax_ids is None:
+                    failed_batches[protein] += 1
+                    if failed_batches[protein] >= args.retries:
+                        logger.warning(
+                            f"Ran out of connection reattempt for protein accession {protein} while fetching Tax IDs\n"
+                            "Therefore not retrieve taxonomy data for this protein."
+                        )
+                        del failed_batches[protein]
+                    continue
+                
                 tax_ids = tax_ids.union(new_tax_ids)
-                protein_ncbi_ids.update(new_prot_ids)
 
-                try:
-                    if failed_individuals[prot] >= args.retries:
-                        del failed_individuals[prot]
-                        failed_proteins.add(prot)
-                except TypeError:
-                    if failed_individuals[str(prot)]['tries'] >= args.retries:
-                        del failed_individuals[str(prot)]
-                        failed_proteins.add(str(prot))
+    failed_prot_accs = [_ for _ in protein_accessions if _ not in set(protein_ncbi_ids.values())]
 
-    if len(failed_proteins) != 0:
-        logger.warning(f"Failed to retrieve lineage data for {len(failed_proteins)} proteins")
-        with open((cache_dir / "failed_protein_accs.txt"), "a") as fh:
-            for prot in failed_proteins:
-                fh.write(f"{prot}\n")
+    if len(failed_prot_accs) != 0:
+        cache_file = cache_dir / "failed_to_retrieve_ids"
+        logger.warning(
+            f"Attempted to retrieve Protein IDs for {len(protein_accessions)} proteins accessions\n"
+            f"Retrieved {len(list(protein_ncbi_ids.keys()))} protein IDs\n"
+            f"Therefore, protein IDs were not retrieved for {len(failed_prot_accs)} protein accessions\n"
+            f"Writing accessions to cache file:{cache_file}"
+        )
 
-    logger.info(f"Retrieved {len(tax_ids)} NCBI Taxonomy IDs")
+        with open(cache_file,'w') as fh:
+            for acc in failed_prot_accs:
+                fh.write(f"{acc}\n")
+    
+    else:
+        logger.warning(
+            f"Success: Attempted to retrieve Protein IDs for {len(protein_accessions)} proteins accessions\n"
+            f"and retrieved {len(list(protein_ncbi_ids.keys()))} protein IDs\n"
+            f"Therefore, retrieved a protein ID for all protein accessions"
+        )
 
     return tax_ids, protein_ncbi_ids
 
 
-def get_ids_from_ncbi(prots, args, failed_batches):
-    """Retrieve NCBI Taxonomy and Protein database IDs, for first time or retried attempts.
+def get_prot_ids(query_key, web_env, args):
+    """Retrieve protein records ids from the NCBI Protein database
 
-    :param protein_accessions: list of NCBI protein version accessions
-    :param args: cmd-line args parser
-    :param round: int, (1) first time, (2) second, (3) continued until reach retry limit
-    :param failed_batches: dict {batch/prots: #ofTries}
+    :param query_key: str, query key from entrez.epost of protein accessions
+    :param web_env: str, web environment from entrez.epost of protein accessions
+    :param args: CLI args parser
 
-    Return set of NCBI Tax ids and set of NCBI Prot ids, and failed_batches dict
+    Return dict {protein ID: protein accession} or None if fails
     """
     logger = logging.getLogger(__name__)
-
-    new_tax_ids, new_protein_ids = set(), set()
-
-    # post protein accessions
+    new_prot_ids = {}  # {id: acc}
     try:
-        query_key, web_env = post_ids(prots, 'Protein', args)
-    except RuntimeError:
-        logger.warning(f"Batch contained invalid protein version accession.\nBatch:\n{prots}")
-        key = str(prots)
-        try:
-            failed_batches[key]['tries'] += 1
-        except KeyError:
-            failed_batches[key] = {'proteins': prots, 'tries': 1}
+        with entrez_retry(
+            args.retries,
+            Entrez.efetch,
+            query_key=query_key,
+            WebEnv=web_env,
+            db="Protein",
+            rettype='docsum',
+        ) as handle:
+            prot_records = Entrez.read(handle, validate=False)
 
-        return new_tax_ids, new_protein_ids, failed_batches
-
-    if query_key is None:
-        key = str(prots)
-        try:
-            failed_batches[key]['tries'] += 1
-        except KeyError:
-            failed_batches[key] = {'proteins': prots, 'tries': 1}
-
-        return new_tax_ids, new_protein_ids, failed_batches
-
-    # elink proteins to tax records
-    new_tax_ids, new_protein_ids, success = link_prot_taxs(query_key, web_env, args)
-
-    if success is False:
+    except (TypeError, AttributeError, RuntimeError,  NotXMLError, IncompleteRead) as err:
         logger.warning(
-            f"Could not retrieve tax link for {','.join(prots)}."
+            "Could not rerieve records from the NCBI Protein database.\n"
+            "Will retry individual records later\n"
+            f"Error message retrieved:\n{err}"
         )
-        key = str(prots)
-        try:
-            failed_batches[key]['tries'] += 1
-        except KeyError:
-            failed_batches[key] = {'proteins': prots, 'tries': 1}
+        return None
 
-        return new_tax_ids, new_protein_ids, failed_batches
+    for record in prot_records:
+        prot_id = record['Id']
+        prot_acc = record['AccessionVersion']
+        new_prot_ids[prot_id] = prot_acc
 
-    new_prots_id_dict = {}  # protein ID: protein ACC
-
-    for i in range(len(new_protein_ids)):
-        prot_acc = prots[i]
-        prot_id = new_protein_ids[i]
-        new_prots_id_dict[prot_id] = prot_acc
-
-    return new_tax_ids, new_prots_id_dict, failed_batches
+    return new_prot_ids
 
 
-def link_prot_taxs(query_key, web_env, args):
+def link_prot_taxs(query_key, web_env, args, batch):
     """Get NCBI tax and protein IDs from elinking protein accs to tax records
 
     :param query_key: str, from Entrez epost
     :param web_env: str, from Entrez epost
     :param args: cmd-line args parser
+    :param batch: list of ncbi protein accs - used in error messages only
 
-    Return set of NCBI tax ids, and set of NCBI protein IDS, and bool TRUE if successful
+    Return set of NCBI tax ids or None if fails
     """
     logger = logging.getLogger(__name__)
 
-    tax_ids, protein_ids = set(), set()
+    tax_ids = set()
 
     try:
         with entrez_retry(
@@ -530,21 +574,31 @@ def link_prot_taxs(query_key, web_env, args):
             linkname="protein_taxonomy",
         ) as handle:
             tax_links = Entrez.read(handle, validate=False)
-    except RuntimeError:
-        logger.warning("Batch included invalid NCBI protein ids")
-        return tax_ids, protein_ids, False
+    except RuntimeError as err:
+        logger.warning(
+            "When retrieving taxonomy IDs, the batch included invalid NCBI protein ids\n"
+            "Will try batch components individually later, to identify the offending id\n"
+            f"Batch:\n{batch}\n"
+            f"Error:\n{err}"
+        )
+        return None
+    except (TypeError, AttributeError, NotXMLError, IncompleteRead) as err:
+        logger.warning(
+            "Failed to connect to NCBI.Entrez and download the Entrez.elink result\n"
+            "Will retry later"
+            f"Batch:\n{batch}\n"
+            f"Error:\n{err}"
+        )
+        return None
+
 
     for result in tax_links:
-
-        for prot_id in result['IdList']:
-            protein_ids.add(prot_id)
-
         for item in result['LinkSetDb']:
             links = item['Link']
             for link in links:
                 tax_ids.add(link['Id'])
 
-    return tax_ids, list(protein_ids), True
+    return tax_ids
 
 
 def get_lineage_protein_data(tax_ids, prot_id_dict, gbk_dict, cache_dir, args):
@@ -560,51 +614,34 @@ def get_lineage_protein_data(tax_ids, prot_id_dict, gbk_dict, cache_dir, args):
     """
     logger = logging.getLogger(__name__)
     tax_prot_dict = {}  # {ncbi tax id: {phylo_rank: str, proteins: [NCBI protein IDs]}}
+    all_failed_ids = set()  # tax ids for whom data could not be retrieved
 
     # retrieve lineage data from the NCBI Taxonomy database
+    lineage_dict, failed_ids = get_lineage_data(tax_ids, args)
+    logger.warning(
+        f"Queried NCBI with {len(tax_ids)} tax ids\n"
+        f"Retrieved lineages for {len(list(lineage_dict.keys()))} tax ids"
+    )
 
-    failed_lineage_ids = {}
-    lineage_dict = {}  # {ncbi tax id: {rank: str}}
-    completely_failed_tax_ids = set()
+    if len(list(lineage_dict.keys())) == 0:
+        return lineage_dict
 
-    for tax_id in tqdm(tax_ids, desc="Retrieving lineages from NCBI"):
-        # get lineage information
-        lineage_dict, success = get_lineage(tax_id, lineage_dict, args)
-
-        if success is False:
-            failed_lineage_ids[tax_id] = 1  # marks the one failed attempt has been retried
-
-    if len(failed_lineage_ids) != 0:
-        logger.info(f"Retrying retrieving lineage data for failed NCBI Tax IDs")
-        while len(list(failed_lineage_ids.keys())) > 0:
-            for tax_id in tqdm(failed_lineage_ids, desc="Retrying retrieving NCBI lineages"):
-                lineage_dict, success = get_lineage(tax_id, lineage_dict, args)
-
-                if success is False:
-                    failed_lineage_ids[tax_id] += 1  # number of attempts that have failed
-
-                if failed_lineage_ids[tax_id] > args.retries:
-                    logger.warning(
-                        f"Ran out of reattempts to get lineage data for ncbi tax {tax_id}"
-                    )
-                    del failed_lineage_ids[tax_id]
-                    completely_failed_tax_ids.add(f"{tax_id}\tCould not retrieve lineage data")
+    all_failed_ids = all_failed_ids.union(failed_ids)
 
+    # cache the retrieved lineages
     with open((cache_dir/"ncbi_lineages.json"), "w") as fh:
         json.dump(lineage_dict, fh)
 
-    # retrieve proteins linked to taxon record in NCBI
-    tax_prot_dict = {}
+    # retrieve proteins linked to taxon record in NCBI - for only tax ids where lineage data was retrieved
+    tax_prot_dict = {}  # {tax_id: {local db protein ids}}
     failed_linked_ids = {}
 
-    for tax_id in tqdm(tax_ids, desc="Link proteins to NCBI Tax record"):
-        if tax_id in completely_failed_tax_ids:
-            logger.warning(
-                f"Could not retrieved lineage for ncbi tax id {tax_id} "
-                "so not retrieving linked proteins for tax id"
-            )
-            continue
-        tax_prot_dict, success = get_tax_proteins(
+    # must query each tax id individually
+    # Batch querying Entrez.elink combines all the protein ids together for all tax ids
+    # therefore, could not separate out the protein ids for each tax id
+    # batch query returns ... DbFrom': 'taxonomy', 'IdList': ['55207', '204038', '1421545']}]
+    for tax_id in tqdm(lineage_dict, desc="Link proteins to NCBI Tax record"):    
+        new_tax_prot_dict, success = get_tax_proteins(
             tax_id,
             tax_prot_dict,
             prot_id_dict,
@@ -613,6 +650,7 @@ def get_lineage_protein_data(tax_ids, prot_id_dict, gbk_dict, cache_dir, args):
             args,
         )
         # {tax_id: {local db protein ids}}
+        tax_prot_dict.update(new_tax_prot_dict)        
 
         if success is False:
             failed_linked_ids[tax_id] = 1  # first attempt to conenct failed
@@ -638,7 +676,7 @@ def get_lineage_protein_data(tax_ids, prot_id_dict, gbk_dict, cache_dir, args):
                         f"Ran out of reattempts to get linked proteins data for ncbi tax {tax_id}"
                     )
                     del failed_linked_ids[tax_id]
-                    completely_failed_tax_ids.add(f"{tax_id}\tCould not retrieve linked proteins")
+                    all_failed_ids.add(f"{tax_id}\tCould not retrieve linked proteins")
 
     # combine lineage data and proteins into a single dict
     for tax_id in tax_ids:
@@ -648,105 +686,19 @@ def get_lineage_protein_data(tax_ids, prot_id_dict, gbk_dict, cache_dir, args):
             logger.error(
                 f"Did not retrieve lineage and/or linked proteins for ncbi tax id {tax_id}"
             )
-            if tax_id not in completely_failed_tax_ids:
-                completely_failed_tax_ids.add(
+            if tax_id not in all_failed_ids:
+                all_failed_ids.add(
                     f"{tax_id}\tCould not lineage data and/or retrieve linked proteins"
                 )
 
-    if len(completely_failed_tax_ids) != 0:
+    if len(all_failed_ids) != 0:
         with open((cache_dir / "failed_tax_ids.txt"), "a") as fh:
-            for tax_id in completely_failed_tax_ids:
+            for tax_id in all_failed_ids:
                 fh.write(f"{tax_id}\n")
 
     return lineage_dict
 
 
-def get_lineage(tax_id, lineage_dict, args):
-    """Retrieve lineage from NCBI taxonomy record, and add to lineage dict
-
-    :param tax_id: str, ncbi tax db id
-    :param lineage_dict: dict, {ncbi tax ID: {rank: str}}
-    :param args: cmd-line args parser
-
-    Return dict of lineage data and bool representing if eFetch was performed successfully
-    """
-    logger = logging.getLogger(__name__)
-
-    try:
-        with entrez_retry(
-            args.retries,
-            Entrez.efetch,
-            db="Taxonomy",
-            id=tax_id,
-        ) as handle:
-            tax_records = Entrez.read(handle, validate=False)
-
-    except (TypeError, AttributeError) as err:
-        logger.warning(f"Failed to fetch tax record from NCBI tax for id '{tax_id}'':\n{err}")
-        return lineage_dict, False
-
-    for record in tax_records:
-        record_id = record['TaxId']
-
-        # set lineage data to None
-        kingdom, phylum, tax_class, order, family, genus, species, strain = None, None, None, None, None, None, None, None
-
-        for i in record['LineageEx']:
-            rank = i['Rank']
-
-            if rank == 'superkingdom':
-                kingdom = i['ScientificName']
-
-            elif rank == 'phylum':
-                phylum = i['ScientificName']
-
-            elif rank == 'class':
-                tax_class = i['ScientificName']
-
-            elif rank == 'order':
-                order = i['ScientificName']
-
-            elif rank == 'family':
-                family = i['ScientificName']
-
-            elif rank == 'genus':
-                genus = i['ScientificName']
-
-            elif rank == 'species' or 'species group':
-                species = i['ScientificName']
-
-            elif rank == 'serotype' or 'strain':
-                strain = i['ScientificName']
-
-        scientific_name = record['ScientificName']
-
-        if genus is not None:
-            # drop genus from species name
-            if species is not None:
-                species = species.replace(genus, "").strip()
-
-            # extract strain from scientific name if not retrieved as rank
-            if species is not None and strain is None:
-                strain = scientific_name.replace(f"{genus} {species}", "").strip()
-
-            # extract species from the scientific name if not retrieved as rank
-            elif species is None:
-                species = scientific_name.replace(genus, "").strip()
-
-        lineage_dict[record_id] = {
-            'kingdom': kingdom,
-            'phylum': phylum,
-            'class': tax_class,
-            'order': order,
-            'family': family,
-            'genus': genus,
-            'species': species,
-            'strain': strain,
-        }
-
-    return lineage_dict, True
-
-
 def get_tax_proteins(tax_id, tax_prot_dict, prot_id_dict, gbk_dict, cache_dir, args):
     """Get the proteins linked to a tax id in NCBI, and link the tax id with the local db protein ids
 
@@ -810,5 +762,102 @@ def get_tax_proteins(tax_id, tax_prot_dict, prot_id_dict, gbk_dict, cache_dir, a
     return tax_prot_dict, True
 
 
+def check_for_failed_accessions(tax_prot_dict, gbk_dict, cache_dir):
+    """Check to see for how many proteins, taxonomy inforamtion could not be retrieved from NCBI
+    
+    :param tax_prot_dict: {tax_id: {linaege info, 'proteins' {local db protein ids}}
+    :param gbk_dict: dict of ncbi accessions paired with local db IDs
+    :param cache_dir: Path to cache directory
+    
+    Return list of accessions for whom tax data was not retrieved from ncbi
+    """
+    logger = logging.getLogger(__name__)
+    protein_accessions = list(gbk_dict.keys())
+
+    # tax_prot_dict = {tax_id: {linaege info, 'proteins' {local db protein ids}}
+    retrieved_protein_acc = []
+    for tax_id in tax_prot_dict:
+        for protein in tax_prot_dict[tax_id]['proteins']:
+            db_id = protein
+            for acc in gbk_dict:
+                if gbk_dict[acc] == db_id:
+                    protein_acc = acc
+                    break
+            
+            retrieved_protein_acc.append(protein_acc)
+
+    failed_accessions = [_ for _ in protein_accessions if _ not in retrieved_protein_acc]
+    if len(failed_accessions) != 0:
+        cache_file = cache_dir / "failed_to_retrieve_tax_data"
+        logger.warning(
+            f"Attempted to retrieve tax data for {len(protein_accessions)} proteins accessions\n"
+            f"Retrieved tax data for {len(retrieved_protein_acc)} proteins\n"
+            f"Therefore, tax data was not retrieved for {len(failed_accessions)} protein accessions\n"
+            f"Writing accessions to cache file:{cache_file}"
+        )
+
+        with open(cache_file,'w') as fh:
+            for acc in failed_accessions:
+                fh.write(f"{acc}\n")
+    
+    else:
+        logger.warning(
+            f"Success: Attempted to retrieve tax data for {len(protein_accessions)} proteins accessions\n"
+            f"and retrieved tax data for {len(retrieved_protein_acc)} proteins"
+        )
+
+    return failed_accessions
+
+
+def retry_failed_linkages(
+    class_filters,
+    family_filters,
+    kingdom_filters,
+    taxonomy_filter_dict,
+    ec_filters,
+    connection,
+    prot_id_dict,
+    cache_dir,
+    tax_prot_dict,
+    args,
+):
+    """Retry connecting proteins to taxonomy as single linkages
+    Used mostly for debugging retrieval of taxonomy info for all protein accessions
+    """
+    logger = logging.getLogger(__name__)
+    if gbk_dict is None:
+        logger.warning("Loading Genbanks table into dict to check for failed retrievals of tax data")
+        gbk_dict = get_db_proteins(
+            class_filters,
+            family_filters,
+            kingdom_filters,
+            taxonomy_filter_dict,
+            ec_filters,
+            connection,
+            args,
+        )
+        # gbk_dict = {gbk acc: db id}
+
+    # check to see for how many proteins a ncbi taxonomy record could not be retrieved
+    failed_accessions = check_for_failed_accessions(tax_prot_dict, gbk_dict, cache_dir)
+
+    if len(failed_accessions) != 0:
+        # check if failure was due to not getting a protein id from ncbi
+        
+        if prot_id_dict is not None:
+            failed_ids = [_ for _ in failed_accessions if _ not in list(prot_id_dict.values())]
+        else:
+            failed_ids = []
+        
+        if len(failed_ids) != 0:
+            logger.warning(f"Did not retrieved protein IDs for {len(failed_ids)} proteins:\n{failed_ids}")
+
+        # tax_prot_dict = {tax_id: {linaege info, 'proteins' {local db protein ids}}
+        # prot_id_dict = {ncbi prot id: prot acc}
+        tax_prot_dict = retry_tax_retrieval(failed_accessions, failed_ids, prot_id_dict, tax_prot_dict, args, cache_dir)
+
+    return tax_prot_dict
+
+
 if __name__ == "__main__":
     main()
