@@ -59,6 +59,7 @@ def build_parser(argv: Optional[List] = None):
     parser.add_argument(
         "email",
         type=str,
+        nargs='?',
         help="User email address. Requirement of Entrez, used to get source organsism data. Email is not stored be cazy_webscraper."
     )
 
@@ -196,6 +197,13 @@ def build_parser(argv: Optional[List] = None):
         help="When called, content in the existing out dir is NOT deleted",
     )
 
+    parser.add_argument(
+        "--ncbi_batch_size",
+        type=int,
+        default=200,
+        help="Number of genbank accessions in each NCBI Taxonomy db batch query"
+    )
+
     # Add option to not delete content in the existing cache dir
     parser.add_argument(
         "--nodelete_cache",
@@ -222,6 +230,18 @@ def build_parser(argv: Optional[List] = None):
         help="Number of times to retry scraping a family or class page if error encountered",
     )
 
+    parser.add_argument(
+        "--skip_ncbi_tax",
+        dest="skip_ncbi_tax",
+        action="store_true",
+        default=False,
+        help=(
+            "Skip retrieving the latest tax classification from the NCBI Taxonomy db for proteins\n"
+            "listed with multiple taxs in CAZy.\n"
+            "For these proteins the first taxonomy listed in CAZy is added to the local CAZyme db"
+        ),
+    )
+
     # Add option to force file over writting
     parser.add_argument(
         "--sql_echo",
